package com.zskk.dicom.monitor.uploader;

import com.zskk.dicom.config.OSSConfig;
import com.zskk.dicom.monitor.config.Configs;
import com.zskk.dicom.monitor.queue.IQueue;
import com.zskk.dicom.monitor.report.ErrReporter;
import com.zskk.dicom.monitor.utils.ExceptionUtil;
import com.zskk.dicom.oss.LCOSSFileAndCallbackHleper;
import com.zskk.dicom.oss.OSSAddUploadHleper;
import com.zskk.dicom.oss.OSSFileAndCallbackHleper;
import com.zskk.dicom.utils.TYNetUtils;
import org.apache.http.util.TextUtils;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.UUID;

/**
 *
 * 上传辅助类 上传到阿里云oss或服务器fastdfs
 *
 * @param:
 * @return: 
 * @auther: MACHENOIKE
 * @date: 2019/3/21 10:01
 */
public class BaseFileUploader {

	private static final String ARCHIVE_WAY_OSS = "oss";
	private static final String ARCHIVE_WAY_DFS = "fastdfs";
	private static final String ARCHIVE_WAY_LCOSS = "lcoss";
	private static final String ARCHIVE_WAY_TY = "tyoos";

	/**
	 * 文件上传
	 * @param filePath 文件路径
	 * @param successQueue 上传成功后文件路径应该放入的队列
	 * @param errorQueue 上传失败后文件路径应该放入的队列
	 * @return
	 */
	public static boolean upload(String  filePath, IQueue<String> successQueue, IQueue<String> errorQueue) {
		String msg = null;
		boolean uploadResult = false;
		File file = null;
		if(TextUtils.isEmpty(filePath)) {
			msg = "filePath is null or empty";
			Configs.sysLog.error(msg);
			ErrReporter.report(msg);
			return false;
		}
		file = new File(filePath);
		if(!file.exists() || !file.isFile()) {
 			msg = "file 不存在 或者 file为文件夹\t filePath:" + filePath;
 			Configs.sysLog.error(msg);
			ErrReporter.report(msg);
			return false;
		}
		try {
			switch (Configs.archiveWay) {
				case ARCHIVE_WAY_OSS:
					if(Configs.ossAddUpload) {
						uploadResult = new OSSAddUploadHleper(Configs.ossEndpoint, OSSConfig.ACCESS_KEY_ID, OSSConfig.ACCESS_KEY_SECRET).upload(file);
					} else  {
						uploadResult = new OSSFileAndCallbackHleper(Configs.ossEndpoint, OSSConfig.ACCESS_KEY_ID, OSSConfig.ACCESS_KEY_SECRET).upload(file);
					}
				break;
				case ARCHIVE_WAY_LCOSS:
					uploadResult = LCOSSFileAndCallbackHleper.upload(new File(filePath));
//					uploadResult = LcUfileUtils.upload(new File(filePath));
				break;
				case ARCHIVE_WAY_TY:
					uploadResult = TYNetUtils.upload(new File(filePath));
					break;
				case ARCHIVE_WAY_DFS:
				default:
					String boundaryKey = UUID.randomUUID().toString().replaceAll("-", "").substring(8, 24);
					String content = "\r\n----" + boundaryKey + "\r\n" + "Content-Type: application/octet-stream\r\n" + "Content-Disposition: form-data; name=\"" + renameFileName(file.getName()) + "\"; filename=\"" + renameFileName(file.getName()) + "\"\r\n" + "Content-Transfer-Encoding: binary\r\n\r\n";
					String postUrl = "http://" + Configs.postHost + ":" + Configs.postPort + Configs.postUri;
					uploadResult = uploadToUrl(postUrl, boundaryKey, content, "", file);
				break;
			}
			if (uploadResult) {
				successQueue.push(filePath);
			} else {
				errorQueue.push(filePath);
			}
		} catch (Exception e) {
			errorQueue.push(filePath);
			Configs.sysLog.warn("上传错误\t" + ExceptionUtil.getExceptionTxt(e));
			ErrReporter.report(ExceptionUtil.getExceptionTxt(e));

		}
		return uploadResult;
	}

	private static String renameFileName(String name) {
		return Configs.hospitalId + "-" + name;
	}

	@SuppressWarnings("static-access")
	private static boolean uploadToUrl(String url, String boundaryKey, String contentBinary, String open, File fromFile) throws Exception {
		String uri = url;
		String enddata = "\r\n----" + boundaryKey + "--";
		// 获取文件输入流
		InputStream in = null;
		try {
			in = new FileInputStream(fromFile);
		} catch (FileNotFoundException fnfe) {
			// 有些图片格式，会被系统临时占用，抛出文件被占用异常
			Thread.currentThread().sleep(1000);
			// 多线程竞争时，文件流可能是空的，读之前做一次判断，以免处理已经上传过的文件(或已被删除)
			if (fromFile.exists()) {
				in = new FileInputStream(fromFile);
			}
		}
		if (in == null) {
			// 多线程竞争时，文件流可能是空的
			// 第二次尝试
			// 有些图片格式，会被系统临时占用，抛出文件被占用异常
			Thread.currentThread().sleep(1000);
			// 多线程竞争时，文件流可能是空的，读之前做一次判断，以免处理已经上传过的文件(或已被删除)
			if (fromFile.exists()) {
				in = new FileInputStream(fromFile);
			}
		}
		if (in == null) {
			// 多线程竞争时，文件流可能是空的
			return false;
		}
		URL urlObj = new URL(uri);
		HttpURLConnection con = (HttpURLConnection) urlObj.openConnection();
		try {
			con.setRequestMethod("POST"); // 设置关键值,以Post方式提交表单，默认get方式
			con.setDoInput(true);
			con.setDoOutput(true);
			con.setUseCaches(false); // post方式不能使用缓存
			// 设置请求头信息
			// con.setRequestProperty("Connection", "Keep-Alive");
			con.setRequestProperty("Charset", "UTF-8");
			con.setRequestProperty("institution", Configs.hospitalId);
			con.setRequestProperty("filename", renameFileName(fromFile.getName()));
			// 设置边界
			con.setRequestProperty("Content-Type", "multipart/form-data; boundary=--" + boundaryKey);
			// 设置文件大小
			con.setRequestProperty("Content-Length", String.valueOf(fromFile.length()));
			// 请求正文信息
			// 第一部分：
			StringBuilder sb = new StringBuilder();
			sb.append(contentBinary);
			// 未知文件类型，以流的方式上传
			byte[] head = sb.toString().getBytes("utf-8");
			// 获得输出流
			OutputStream out = new DataOutputStream(con.getOutputStream());
			// 输出表头
			out.write(head);
			// 文件正文部分
			// 把文件以流文件的方式 推入到url中
			DataInputStream dataIn = new DataInputStream(in);
			int bytes = 0;
			byte[] bufferOut = new byte[1024];
			while ((bytes = dataIn.read(bufferOut)) != -1) {
				out.write(bufferOut, 0, bytes);
			}
			dataIn.close();
			try {
				in.close();
			} catch (Exception e) {
			}
			// 结尾部分
			byte[] foot = (enddata).getBytes("utf-8");// 定义最后数据分隔线
			out.write(foot);
			out.flush();
			out.close();
			// 读取返回数据
			StringBuilder strBuf = new StringBuilder();
			BufferedReader reader = new BufferedReader(new InputStreamReader(con.getInputStream()));
			try {
				String line = null;
				while ((line = reader.readLine()) != null) {
					strBuf.append(line).append("\n");
				}
			} finally {
				reader.close();
			}
			String rs = strBuf.toString();
			Configs.sysLog.info("The file (" + fromFile.getName() + ") uploaded ! result:" + rs);
			if (strBuf.indexOf("success\":true") > 0) {
				return true;
			} else {
				return false;
			}
		} finally {
			try {
				con.disconnect();
				con = null;
			} catch (Exception e) {
			}
			try {
				in.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}


}
